/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.supinfo.spitfireboard.servlets;

import com.supinfo.spitfireboard.entities.Post;
import com.supinfo.spitfireboard.entities.User;
import com.supinfo.spitfireboard.helpers.Password;
import com.supinfo.spitfireboard.services.UserService;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.EJB;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

/**
 *
 * @author Fabien LOISON
 */
@ManagedBean
@RequestScoped
public class ProfileController {

    @EJB
    private UserService userService;

    private String getValue(String fieldName) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        return (String) facesContext.getExternalContext().getRequestParameterMap().get(fieldName);
    }

    /**
     * Creates a new instance of ProfileController
     */
    public ProfileController() {
    }

    private Long getUserId() {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        String idStr = (String) facesContext.getExternalContext().getRequestParameterMap().get("id");
        if (idStr == null) {
            return null;
        }
        return Long.valueOf(idStr);
    }

    public User getUser() {
        return userService.getUser(this.getUserId());
    }

    public List<Post> getLatestPosts() {
        return userService.getUserLatestPost(this.getUserId());
    }

    public List<String> doEdit() {
        List<String> errors = new ArrayList<String>();

        Long userId = null;
        
        if (this.getValue("username").equals("")) {
            errors.add("The 'Username' field is empty");
        }
        if (this.getValue("email").equals("")) {
            errors.add("The 'Email' field is empty");
        }

        if (errors.isEmpty()) {
            try {
                userId = userService.userEdit(Long.valueOf(this.getValue("userId")), this.getValue("username"), this.getValue("email"), this.getValue("avatar"));
            } catch (Exception ex) {
                errors.add("Oops! Something wrong appened.");
                return errors;
            }

            // Redirect
            ExternalContext context = FacesContext.getCurrentInstance().getExternalContext();
            try {
                context.redirect("profile.xhtml?id=" + userId);
            } catch (IOException ex) {
                Logger.getLogger(TopicController.class.getName()).log(Level.SEVERE, null, ex);
            }

        } else {
            return errors;
        }
        return null;
    }
    public List<String> doDelete() {
        List<String> errors = new ArrayList<String>();

        Long userId = null;

        if (errors.isEmpty()) {
            try {
                userService.deleteUser(Long.valueOf(this.getValue("userId")));
            } catch (Exception ex) {
                errors.add("Oops! Something wrong appened.");
                return errors;
            }

            // Redirect
            ExternalContext context = FacesContext.getCurrentInstance().getExternalContext();
            try {
                context.redirect("index.xhtml");
            } catch (IOException ex) {
                Logger.getLogger(TopicController.class.getName()).log(Level.SEVERE, null, ex);
            }

        } else {
            return errors;
        }
        return null;
    }

    public List<String> doPasswordEdit() {
        List<String> errors = new ArrayList<String>();

        Long userId = null;

        if (this.getValue("current_password").equals("")) {
            errors.add("The 'Current Password' field is empty");
        }
        if (this.getValue("password").equals("")) {
            errors.add("The 'New Password' field is empty");
        }
        if (this.getValue("password2").equals("")) {
            errors.add("The 'New Password (again)' field is empty");
        }

        // Check if both passwords are equal
        if (!this.getValue("password").equals(this.getValue("password2"))) {
            errors.add("The two passwords are not equal.");
        }

        if (!userService.getUser(Long.valueOf(this.getValue("userId"))).getPassword().equals(Password.hash(this.getValue("current_password")))) {
            errors.add("The current password is incorrect.");
            return errors;
        }

        // No error...
        if (errors.isEmpty()) {
            try {
                userId = userService.changePassword(Long.valueOf(this.getValue("userId")), this.getValue("password"));
                // Redirect
                ExternalContext context = FacesContext.getCurrentInstance().getExternalContext();
                try {
                    context.redirect("profile.xhtml?id=" + userId);
                } catch (IOException ex) {
                    Logger.getLogger(TopicController.class.getName()).log(Level.SEVERE, null, ex);
                }
            } catch (Exception ex) {
                errors.add("Oops! Something wrong appened.");
                return errors;
            }
        } else {
            return errors;
        }

        return null;
    }

    public List<String> doRoleEdit() {
        List<String> errors = new ArrayList<String>();

        Long userId = null;

        try {
            userId = userService.changeRole(Long.valueOf(this.getValue("userId")), Integer.valueOf(this.getValue("role")));
        } catch (Exception ex) {
            errors.add("Oops! Something wrong appened.");
            return errors;
        }

        // Redirect
        ExternalContext context = FacesContext.getCurrentInstance().getExternalContext();
        try {
            context.redirect("profile.xhtml?id=" + userId);
        } catch (IOException ex) {
            Logger.getLogger(TopicController.class.getName()).log(Level.SEVERE, null, ex);
        }

        return null;
    }
}
